import dbus

NMI = 'org.freedesktop.NetworkManager'
PI = 'org.freedesktop.DBus.Properties'
SSC = "org.freedesktop.NetworkManagerSystemSettings"
USC = "org.freedesktop.NetworkManagerUserSettings"
DEV = "org.freedesktop.NetworkManager.Device"
IP4C = 'org.freedesktop.NetworkManager.IP4Config'

def get_ipaddresses():
    obj = dbus.SystemBus().get_object(NMI, '/org/freedesktop/NetworkManager')
    nmProp = dbus.Interface(obj, dbus_interface=PI)
    activeConnections = nmProp.Get(NMI, 'ActiveConnections')
    devices = []
    for conPath in activeConnections:
        objCon = dbus.SystemBus().get_object(NMI, conPath)
        devices += objCon.Get('org.freedesktop.NetworkManager.Connection.Active', 'Devices', dbus_interface=PI)
    addresses = []
    for devicePath in devices:
        objDevice = dbus.SystemBus().get_object(NMI, devicePath)    
        ip4configPath = objDevice.Get(DEV, 'Ip4Config', dbus_interface=PI)
        if ip4configPath != '/':
            obj_ip4config = dbus.SystemBus().get_object(NMI, ip4configPath)
            addresses += obj_ip4config.Get(IP4C, 'Addresses', dbus_interface=PI)
    if addresses != None:
        return addresses
    else:
        return ()  

def list_connections():
    res = []
    for svc in [SSC, USC]:
        so = dbus.SystemBus().get_object(svc, '/org/freedesktop/NetworkManagerSettings')
        si = dbus.Interface(so, 'org.freedesktop.NetworkManagerSettings') 
        for opath in si.ListConnections():
            co = dbus.SystemBus().get_object(svc, opath)
            ci = dbus.Interface(co, 'org.freedesktop.NetworkManagerSettings.Connection')
            res += [str(ci.GetSettings()['connection']['id'])]
    return res
    
def list_interfaces():
    obj = dbus.SystemBus().get_object(NMI, '/org/freedesktop/NetworkManager')
    nm = dbus.Interface(obj, dbus_interface=NMI)
    devices = nm.GetDevices()
    res = []
    for opath in nm.GetDevices():
        device = dbus.SystemBus().get_object(NMI, opath)
        deviceProp = dbus.Interface(device, dbus_interface=PI)
        iface = deviceProp.Get(DEV, 'Interface')
        res += [str(iface)]
    return res

def get_ips_of_interface(iface):
    #@return: None or [(ip, mask, gateway), ...]
    #find device by iface in active connections
    obj = dbus.SystemBus().get_object(NMI, '/org/freedesktop/NetworkManager')
    nmProp = dbus.Interface(obj, dbus_interface=PI)
    activeConnections = nmProp.Get(NMI, 'ActiveConnections')
    devices = []
    for conPath in activeConnections:
        objCon = dbus.SystemBus().get_object(NMI, conPath)
        devices += objCon.Get('org.freedesktop.NetworkManager.Connection.Active', 'Devices', dbus_interface=PI)
    addresses = []
    for devicePath in devices:
        objDevice = dbus.SystemBus().get_object(NMI, devicePath)
        device_iface = str(objDevice.Get(DEV, 'Interface', dbus_interface=PI))
        #if device founded, get IP4Config of device
        if device_iface == iface: 
            ip4configPath = objDevice.Get(DEV, 'Ip4Config', dbus_interface=PI)
            if ip4configPath != '/':
                obj_ip4config = dbus.SystemBus().get_object(NMI, ip4configPath)
                addresses += obj_ip4config.Get(IP4C, 'Addresses', dbus_interface=PI)
    if addresses != None:
        return addresses
    else:
        return ()  
    
