#!/usr/bin/env python

#TODO: testing
#TODO: git push
#TODO: call Paul, ask him about xl2tp
#TODO: improve disonnect notification

import dbus
import dbus.service
import gobject
import logging
import syslog
from dbus.mainloop.glib import DBusGMainLoop
import os
import traceback


from syslogger import MultiLogger
import openswan
import nmi
import nmvpnpluginservice as nmvpn
from routine import *

logger = MultiLogger("NMVPNOpenswanService", 0)
nmvpn.logger = logger

service_name = "org.freedesktop.NetworkManager.openswan"


class ConfigLocation:
    IPSEC_CONF = "/etc/ipsec.conf"
    IPSEC_SECRETS = "/etc/ipsec.secrets"
    NM_IPSEC_CONF = "/etc/ipsec.d/nm/ipsec.conf"
    NM_IPSEC_SECRETS = "/etc/ipsec.d/nm/ipsec.secrets"
    IPSEC_CONF_SETUP_OLD = "/etc/ipsec.d/nm/ipsec.conf.setup.old"

def get_nm_ipsec_conf():
    # create nm/ipsec.conf
    if not os.path.exists(ConfigLocation.NM_IPSEC_CONF):
        create_empty_file(ConfigLocation.NM_IPSEC_CONF)
    os.chown(ConfigLocation.NM_IPSEC_CONF, 0, 0)
    os.chmod(ConfigLocation.NM_IPSEC_CONF, 0600)
    #
    # include nm/ipsec.conf to main ipsec.conf
    ipsec_conf = openswan.IpsecConf(ConfigLocation.IPSEC_CONF)
    ipsec_conf.include(ConfigLocation.NM_IPSEC_CONF)
    # add alsoflip to config setup
    # ipsec_conf.get_section('setup').alsoflip('setup-nm')
    ipsec_conf.save()
    #
    # return included nm/ipsec.conf
    return openswan.IpsecConf(ConfigLocation.NM_IPSEC_CONF)
    
def get_nm_ipsec_secrets():
    # create nm/ipsec.secrets
    if not os.path.exists(ConfigLocation.NM_IPSEC_SECRETS):
        create_empty_file(ConfigLocation.NM_IPSEC_SECRETS)
    os.chown(ConfigLocation.NM_IPSEC_SECRETS, 0, 0)
    os.chmod(ConfigLocation.NM_IPSEC_SECRETS, 0600)
    #
    # include nm/ipsec.secrets to main ipsec.secrets
    ipsec_secrets = openswan.IpsecSecrets(ConfigLocation.IPSEC_SECRETS)
    ipsec_secrets.include(ConfigLocation.NM_IPSEC_SECRETS)
    ipsec_secrets.save()
    #
    # return included nm/ipsec.secrets
    return openswan.IpsecSecrets(ConfigLocation.NM_IPSEC_SECRETS)

def copy_ipsec_conf_setup():
    # create nm/ipsec.conf.setup.old
    if not os.path.exists(ConfigLocation.IPSEC_CONF_SETUP_OLD):
        create_empty_file(ConfigLocation.IPSEC_CONF_SETUP_OLD)
    os.chown(ConfigLocation.NM_IPSEC_SECRETS, 0, 0)
    os.chmod(ConfigLocation.NM_IPSEC_SECRETS, 0600)
    #
    # open ipsec.conf and copy "conf setup"-section to nm/ipsec.conf.setup.old
    ipsec_conf = openswan.IpsecConf(ConfigLocation.IPSEC_CONF)
    ipsec_conf_setup = ipsec_conf.get_section('setup')
    reserv = openswan.IpsecConf(ConfigLocation.IPSEC_CONF_SETUP_OLD)
    reserv.clear()
    reserv.add_section(ipsec_conf_setup)
    reserv.save()
    
def get_ipsec_conf():
    return openswan.IpsecConf(ConfigLocation.IPSEC_CONF)
    
def restore_ipsec_conf_setup():
    if not os.path.exists(ConfigLocation.IPSEC_CONF_SETUP_OLD):
        return
    reserv = openswan.IpsecConf(ConfigLocation.IPSEC_CONF_SETUP_OLD)
    reserv_setup = reserv.get_section('setup')
    ipsec_conf = openswan.IpsecConf(ConfigLocation.IPSEC_CONF)
    ipsec_conf_setup = ipsec_conf.get_section('setup')
    keys = ipsec_conf_setup.keys()
    for param in keys:
        if not reserv_setup.has_key(param):
            del ipsec_conf_setup[param]
        else:
            ipsec_conf_setup[param] = reserv_setup[param]
    ipsec_conf.save()
    
# data:
#   type: "Host-to-Host", "Host-to-Network"
#   gateway: string
#   remotenetwork: string
#   mask: string
#   auth: "X509", "RSA"

                
class NmOpenswanService(nmvpn.NmVpnPluginService):
    def __init__(self):
        global service_name
        self.conn = None
        nmvpn.NmVpnPluginService.__init__(self, service_name)
        self._init_()
        
    def _init_(self):
        self.ip4reply_sent = True
        self.set_state(nmvpn.ServiceState.INIT)        
        self.set_state(nmvpn.ServiceState.STARTING)
        #starting ipsec
        self.ipsec = openswan.IpsecControl()
        self.ipsec.start()
        #all done
        self.set_state(nmvpn.ServiceState.STARTED)
    
    def need_secrets(self, connection):
        data = connection['vpn']['data']
        if data['nss'] == 'no' and \
           (data['auth'] == 'X509' or data['auth'] == 'PSK'):
            return "vpn"
        return ""
        
    def connect(self, connection):
        try:
            self.create_connection(connection['vpn'])
            self.conn.start_async(self.connection_start_cb)
            self.ip4reply_sent = False
        except Exception, e:
            logger.error("Couldn't connect: %s; %s" % (e.message, traceback.format_exc()))        
            raise nmvpn.LaunchFailedError()
    
    def disconnect(self):
        if self.conn != None:
            self.conn.stop()
            restore_ipsec_conf_setup()
    
    @dbus.service.method("com.xelerance.openswan.NetworkManagerPlugin",
                         in_signature="sa{ss}", out_signature="")
    def ChangeStatusMessage(self, connname, status):
        if self.conn.name == connname:
            logger.debug("ChangeStatusMessage: %s %s" % (connname, status))
            if not self.ip4reply_sent and status['tunnel'] == 'up':
                conf = {}
                conf['tundev'] = status['if_stats']
                address = address_to_long(self.conn_info['left'])
                if address != None:
                    conf['address'] = dbus.UInt32(address)
                    #conf['addr'] = 1                
                    #conf['prefix'] = 24
                    #conf['dns'] = conf['address']
                    #conf['nbns'] = conf['address']
                self.Ip4Config(conf)
                self.ip4reply_sent = True            
    
    def create_connection(self, vpn_info):
        ipsec_conf = get_nm_ipsec_conf()
        ipsec_conf.clear()
        ipsec_secrets = get_nm_ipsec_secrets()
        ipsec_secrets.clear()
        copy_ipsec_conf_setup()
        global_ipsec_conf = get_ipsec_conf()
        setup_nm = global_ipsec_conf.get_section('setup')
        data = vpn_info['data']
        secrets = vpn_info['secrets']
        logger.debug("Start create connection")
        if 'iface' in data and data['iface'] != '[Default]':
            ipas = nmi.get_ips_of_interface(data['iface'])
        else:        
            ipas = nmi.get_ipaddresses()
        new_conn = openswan.Section('conn', 'nmconnection')
        new_conn['left'] = long_to_address(ipas[0][0])
        new_conn['right'] = data['gateway']        
        if data['type'] == "Host-to-Network":
            mask = mask_long_to_short(data['mask'])
            new_conn['rightsubnet'] = "%s/%s" % (data['remotenetwork'], mask)
        if data['nss'] == 'yes':
            new_conn['authby'] = 'rsasig'
            if data['auth'] == 'X509':
                new_conn['leftrsasigkey'] = '%%cert'
                new_conn['leftcert'] = vpn_info['user-name'] # add to ipsec_conf
                ipsec_secrets.add_x509(vpn_info['user-name']) # add to ipsec_secrets
        else:
            if data['auth'] == 'X509':
                new_conn['authby'] = 'rsasig'
                new_conn['leftrsasigkey'] = '%%cert'
                #FIXME: Why using leftcert, if using ipsec.secrets
                new_conn['leftcert'] = data['certfilename']
                ipsec_secrets.add_x509(data['certfilename'], secrets['password'])
            elif data['auth'] == 'RSA':
                pass
            elif data['auth'] == 'PSK':
                new_conn['authby'] = 'secret'
                ipsec_secrets.add_psk([new_conn['left'], new_conn['right']], secrets['password'])
        if 'nat' in data and data['nat'] == 'yes':
            setup_nm['nat_traversal'] = 'yes'
        else:
            setup_nm['nat_traversal'] = 'no'
        ipsec_conf.add_section(new_conn)
        self.conn = openswan.Connection('nmconnection')
        self.conn_info = new_conn
        ipsec_conf.save()
        ipsec_secrets.save()
        global_ipsec_conf.save()
        logger.debug("End of create connection")        
        
    def connection_start_cb(self, error):
        if error != None:
            self.Failure(nmvpn.PluginFailure.CONNECT_FAILED)


if __name__ == '__main__':
    try:
        service = NmOpenswanService()
    except Exception, e:
        logger.error(str(e))

